<?php
// $Id$

/**
 * Page to administer shortcuts sets
 *
 * Presents the admin user an overview page with all the shortcuts sets in the system
 *
 * @todo Use draggable tables to control the shortcuts order of appearance
 *
 * @return
 * A table with all shorcuts sets in the system
 */
function toolbar_shortcuts_set_admin() {
  $result = db_query('SELECT * FROM {toolbar_shortcuts_set}');
  $header = array(
    array('data' => t('Name')),
    array('data' => t('Operations'), 'colspan' => '3')
  );

  $rows = array();
  $destination = drupal_get_destination();
  while ($data = db_fetch_object($result)) {
    $row = array(
      check_plain($data->title),
      l(t('list links'), "shortcuts/$data->set_name"),
      l(t('edit set name'), "admin/build/shortcuts/edit/$data->set_name", array('query' => $destination)),
      l(t('delete'), "admin/build/shortcuts/delete/$data->set_name", array('query' => $destination))
    );
    $rows[] = $row;
  }

  if (empty($rows)) {
    $empty_message = t('No shortcuts sets found. !create', array('!create' => l(t('Create a new one.'), 'admin/build/shortcuts/add-set')));
    $rows[] = array(array('data' => $empty_message, 'colspan' => '4'));
  }

  $output .= theme('table', $header, $rows);

  return $output;
}

/**
 * Form to add and edit shortcuts sets
 *
 * This form is used to add and edit shortcuts sets deppending on the context
 *
 * @param $edit
 * An array with the shortcut set 
 *
 * @return
 * A form to add or edit a shortcut set
 */
function toolbar_shortcuts_set_form(&$form_state, $edit = array('title' => '', 'set_name' => NULL)) {
  $form['#set'] = $edit;

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Shortcuts set title'),
    '#default_value' => $edit['title'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify a name for this shortcut set.'),
    '#required' => TRUE,
  );

  if ($edit['set_name']) {
    $form['set_name'] = array('#type' => 'hidden', '#value' => $edit['set_name']);
    $form['submit'] = array('#type' => 'submit', '#value' => t('Update set'));
  }
  else {
    $form['submit'] = array('#type' => 'submit', '#value' => t('Create new set'));
  }

  return $form;
}

/**
 * Submit handler for the toolbar_shortcuts_set_form
 *
 * Submits the form, we use toolbar_shortcuts_save_set() to insert the shortcut set
 * into the database depending on the context
 *
 * @see toolbar_shortcuts_save_set()
 * @see toolbar_shortcuts_set_form()
 *
 * @return
 * A success message if the shortcut et was saved
 */
function toolbar_shortcuts_set_form_submit($form, &$form_state) {
  $set = (object) array(
    'title' => $form_state['values']['title'],
    'set_name' => $form_state['values']['set_name'] ? $form_state['values']['set_name'] : '',
  );
  
  toolbar_shortcuts_save_set($set);

  drupal_set_message(t('The %set_title shortcut set has been created.', array('%set_title' => $set->title)));
  $form_state['redirect'] = 'admin/build/shortcuts';
  return;
}

/**
 * Main function to insert, update or delete a shortcut set
 *
 * We use this function to insert the shortcut set into the database, if we pass parameters
 * then the shortcuts set exists and we determine if we're going to delete or update a record
 *
 * @param $set
 * the shortcut set object
 *
 * @todo
 * Change the set-name of the set on updating and update all links that belong to that set-name
 * This way we then can check for duplicates and prevent them, since now when the user created
 * a set-name 'default' and then changed the name to 'default2' the set-name remains as 'default'
 * and if the user later wants to create a new set 'default' then we'll have to "set-default"
 *
 */
function toolbar_shortcuts_save_set(&$set) { 
  if($set->set_name) {
    if($set->title) {
      // The set-name has to remain the same. But it has to be unique also, so we actually have to change this and all links within.
      $old_set_name = $set->set_name;
      $new_set_name = 'set-'.strtolower(str_replace(' ', '-', $set->title));
      
      $result = db_query("SELECT * FROM {menu_links} WHERE menu_name = '%s'", $old_set_name);
      while($links = db_fetch_object($result)) {
        db_query("UPDATE {menu_links} SET menu_name = '%s' WHERE mlid = %d", $new_set_name, $links->mlid);
      }
      
      $users = db_query("SELECT * FROM {toolbar_shortcuts_users} WHERE set_name = '%s'", $old_set_name);
      while($user = db_fetch_object($users)) {
        db_query("UPDATE {toolbar_shortcuts_users} SET set_name = '%s' WHERE uid = %d", $new_set_name, $user->uid);
      }
      
      db_query("UPDATE {toolbar_shortcuts_set} SET title = '%s', set_name = '%s' WHERE set_name = '%s'", $set->title, $new_set_name, $old_set_name);
    }
    else {
      db_query("DELETE FROM {toolbar_shortcuts_set} WHERE set_name = '%s'", $set->set_name);
    }
  }
  else if(!$set->set_name && $set->title) {
    // If there is nothing in $set->set_name (we sent it empty) then we create it
    // 
    $set_name = 'set-'.strtolower(str_replace(' ', '-', $set->title));
    db_query("INSERT INTO {toolbar_shortcuts_set} (title, set_name) VALUES ('%s', '%s')", $set->title, $set_name);
  }
  else {
    db_query("DELETE FROM {toolbar_shortcuts_set} WHERE title = '%s'", $set->title);
  }
  
  drupal_clear_path_cache();
}

/** 
 * A form to edit a shortcut set
 *
 * If we pass a valid $set_name then we use toolbar_shortcuts_set_load() to load an object in the $set variable
 * then we use the toolbar_shortcuts_set_form() to edit the shortcut et
 *
 * If we pass no valid $set_name then the shortcut doesn't exists and we need to create it
 *
 * @param $set_name
 * The shortcut set name
 *
 * @see toolbar_shortcuts_set_load()
 * @see toolbar_shortcuts_set_form()
 *
 * @return
 * an empty form if we're creating a new shortcut set or
 * a form with the shortcut set values where editing
 */
function toolbar_shortcuts_set_edit($set_name) { 
  if ($set_name) {
    $set = toolbar_shortcuts_set_load($set_name);
    drupal_set_title(t('Edit shortcut set "!shortcut"', array('!shortcut' => check_plain($set['title']))));
    $output = drupal_get_form('toolbar_shortcuts_set_form', $set);
  }
  else {
    $output = drupal_get_form('toolbar_shortcuts_set_form');
  }

  return $output;
}

/**
 * Confirmation page to delete a shortcut set
 *
 * We use this function to present a confirmation page to delete a shortcut set and all links
 * inside that set
 *
 * @param $set_name
 * the shortcut set name
 * we pass the set name of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_set_load()
 * @see toolbar_shortcuts_set_delete()
 * @see toolbar_shortcuts_set_delete_confirm_submit
 *
 * @return
 * a confirmation page to delete a shortcut set
 */
function toolbar_shortcuts_set_delete_confirm($form_state, $set_name) {
  $set = toolbar_shortcuts_set_load($set_name);
  if (user_access('administer shortcuts')) {
    $form['set_name'] = array('#type' => 'value', '#value' => $set_name);
    $output = confirm_form($form,
      t('Are you sure you want to delete the shortcut set %title? All links inside the set will be deleted too and all users who are using this set will be asigned with the default set.', array('%title' => check_plain($set['title']))),
      isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/shortcuts');
  }
  return $output;
}

/**
 * Submit handler to delete a shortcut set
 *
 * Passes the $set_name from the form in toolbar_shortcuts_set_delete_confirm() to
 * toolbar_shortcuts_set_delete() so we can delete the shortcut set
 *
 * @see toolbar_shortcuts_set_delete()
 * @see toolbar_shortcuts_set_delete_confirm()
 */
function toolbar_shortcuts_set_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    toolbar_shortcuts_set_delete($form_state['values']['set_name']);
    $form_state['redirect'] = 'admin/build/shortcuts';
    return;
  }
}

/**
 * Deletes a shortcut set based on the set name and all links related to that set
 *
 * @param $set_name
 * The set name of the shortcut set we want to delete
 *
 * @see toolbar_shortcuts_set_delete_confirm_submit()
 */
function toolbar_shortcuts_set_delete($set_name) {
  // First, assign all users using this set the default set
  $users = db_query("SELECT uid FROM {toolbar_shortcuts_users} WHERE set_name = '%s'", $set_name);
  while($uid = db_fetch_object($users)) {
    $shortcut_set = toolbar_shortcuts_default_set();
    $account = user_load($uid);
    toolbar_shortcuts_set_assign_user($shortcut_set, $account);
  }
  
  // look for all links with this set name
  $result = db_query("SELECT mlid FROM {menu_links} WHERE menu_name = '%s'", $set_name);
  while($link = db_fetch_object($result)) {
    // delete all links with this set name
    drupal_set_message(t('The shortcut !link_title has been deleted.', array('!link_title' => $link->link_title)));
    menu_link_delete($link->mlid);
  }
  // finally, delete the set
  db_query("DELETE FROM {toolbar_shortcuts_set} WHERE set_name = '%s'", $set_name);
  
  drupal_set_message(t('The shortcut set has been deleted.'));
}