<?php
// $Id$

/**
 * Page to administer shortcuts
 *
 * Presents the admin user an overview page with all the shortcuts sets in the system
 *
 * @todo Use draggable tables to control the shortcuts order of appearance
 *
 * @return
 * A table with all shorcuts sets in the system
 */
function toolbar_shortcuts_links_admin($set_name) {
  $set = toolbar_shortcuts_set_load($set_name);
 
  $result = db_query("SELECT * FROM {menu_links} WHERE menu_name = '%s' ORDER BY weight ASC", $set['set_name']);
  
  $header = array(
    array('data' => t('Name')),
    array('data' => t('Operations'), 'colspan' => '2')
  );

  $rows = array();
  $destination = drupal_get_destination();
  while ($data = db_fetch_object($result)) {
    $row = array(
      l(t($data->link_title), check_plain($data->link_path)),
      l(t('edit shortcut'), "shortcuts/edit/{$data->mlid}", array('query' => $destination)),
      l(t('delete'), "shortcuts/{$set['setid']}/delete/{$data->mlid}", array('query' => $destination))
    );
    $rows[] = $row;
  }

  if (empty($rows)) {
    $empty_message = t("No shortcuts found. !create", array("!create" => l(t("Create a new one."), "shortcuts/{$set['set_name']}/add")));
    $rows[] = array(array('data' => $empty_message, 'colspan' => '3'));
  }

  $output .= theme('table', $header, $rows);

  return $output;
}

function toolbar_shortcuts_link_add_form(&$form_state, $set_name, $edit = array('mlid' => NULL)) {
  $form['#shortcut'] = $edit;

  $form['mlid'] = array(
    '#type' => 'hidden',
    '#default_value' => $edit['mlid'] ? $edit['mlid'] : 0,
  );

  $form['link_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Shortcut name'),
    '#default_value' => $edit['link_title'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify a name for this shortcut.'),
    '#required' => TRUE,
  );
  $form['link_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path'),
    '#default_value' => $edit['link_path'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify a path by which this shortcut can be accessed.'),
    '#required' => TRUE,
  );
  
  $form['set_name'] = array(
    '#type' => 'hidden',
    '#default_value' => $edit['set_name'] ? $edit['set_name'] : $set_name,
  );
  $form['weight'] = array(
    '#type' => 'textfield',
    '#title' => t('Weight'),
    '#default_value' => $edit['weight'],
    '#maxlength' => 5,
    '#size' => 5,
    '#description' => t('Specify a weight.'),
    '#required' => TRUE,
  );
  if ($edit['mlid']) {
    $form['mlid'] = array('#type' => 'hidden', '#value' => $edit['mlid']);
    $form['submit'] = array('#type' => 'submit', '#value' => t('Update shortcut'));
  }
  else {
    $form['submit'] = array('#type' => 'submit', '#value' => t('Create new shortcut'));
  }

  return $form;
}


/**
 * Submit handler for the toolbar_shortcuts_link_add_form
 *
 * Submits the form, build the $link object and then save the link using
 * menu_link_save() to insert the shortcut as a link in the menu system
 *
 * @see toolbar_shortcuts_link_add_form()
 * @see toolbar_shortcuts_set_load()
 * @see menu_link_save()
 *
 * @return
 * A success message if the shortcut was saved
 */
function toolbar_shortcuts_link_add_form_submit($form, &$form_state) {
  $set = toolbar_shortcuts_set_load($form_state['values']['set_name']);
  
  $link = array(
    'menu_name' => $set['set_name'],
    'link_path' => $form_state['values']['link_path'],
    'plid' => 0,
    'mlid' => $form_state['values']['mlid'],
    'link_title' => $form_state['values']['link_title'],
    'weight' => $form_state['values']['weight'],
    'module' => 'toolbar',
  );
  
  menu_link_save($link);
  
  drupal_set_message(t('The %link shortcut has been created.', array('%link' => $form_state['values']['link_title'])));
  $form_state['redirect'] = 'shortcuts/'.$form_state['values']['set_name'];
  return;
}


/** 
 * A form to edit a shortcut set
 *
 * If we pass a valid $setid then we use toolbar_shortcuts_load() to load an object in the $shortcut variable
 * then we use the toolbar_shortcuts_admin_form() to edit the shortcut.
 *
 * If we pass no valid $sid then the shortcut doesn't exists and we need to create it
 *
 * @param $sid
 * The shortcut unique id
 *
 * @see toolbar_shortcuts_load()
 * @see toolbar_shortcuts_admin_form()
 *
 * @return
 * an empty form if we're creating a new shortcut or
 * a form with the shortcut values
 */
function toolbar_shortcuts_link_edit($mlid) { 
  if ($mlid) {
    $link = menu_link_load($mlid);
    drupal_set_title(t('Edit shortcut "!shortcut"', array('!shortcut' => check_plain($link['link_title']))));
    $output = drupal_get_form('toolbar_shortcuts_link_add_form', $link['menu_name'], $link);
  }
  else {
    $output = drupal_get_form('toolbar_shortcuts_link_add_form');
  } 
  
  return $output;
}

/**
 * Confirmation page to delete a shortcut
 *
 * We use this function to present a confirmation page to delete a shortcut
 *
 * @param $sid
 * the shortcut unique id
 * we pass the $sid of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_overview()
 *
 * @return
 * a confirmation page to delete a shortcut
 */
function toolbar_shortcuts_link_delete_confirm($form_state, $set_name, $mlid) {
  $link = menu_link_load($mlid);
  if (user_access('administer shortcuts')) {
    $form['mlid'] = array('#type' => 'value', '#value' => $mlid);
    $output = confirm_form($form,
      t('Are you sure you want to delete the shortcut %title?', array('%title' => check_plain($link['link_title']))),
      isset($_GET['destination']) ? $_GET['destination'] : "shortcuts/{$set['set_name']}");
  }
  return $output;
}

/**
 * Submit handler to delete a shortcut
 *
 * Passes the $sid from the form in toolbar_shortcuts_delete_confirm() to
 * toolbar_shortcuts_admin_delete() so we can delete the shortcut
 *
 * @see toolbar_shortcuts_admin_delete_confirm()
 */
function toolbar_shortcuts_link_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    toolbar_shortcuts_link_delete($form_state['values']['mlid']);
    $form_state['redirect'] = "shortcuts/{$set['set_name']}";
    return;
  }
}

/**
 * Deletes a shortcut based on the shortcut unique id
 *
 * @param $sid
 * The $sid of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_admin_delete_confirm_submit()
 */
function toolbar_shortcuts_link_delete($mlid = 0) {
  menu_link_delete($mlid);
  drupal_set_message(t('The shortcut has been deleted.'));
}
